@@ -17,6 +17,7 @@ These dictionaries are used further in the definitions of calibration samples (s
 
 run2_dir = "root://eoslhcb.cern.ch//eos/lhcb/grid/prod/lhcb/LHCb/"
 run1_dir = "root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_Run1_conversion/Reco14_DATA/"
+photon_dir = "root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDGen2/calibration/photon/"
 
 legacy_run2_pidcalib_datasets = {
   'MagDown_2018': [f"{run2_dir}/Collision18/PIDCALIB.ROOT/00109278/0000/00109278_{i:08}_1.pidcalib.root" for i in range(1, 403) ], 
@@ -41,14 +42,14 @@ legacy_run2_pidcalib_jpsi2mumunopt_datasets = {
 }
 
 legacy_run2_b2jpsieek_stripping_datasets = {
-  'MagDown_2015': ['/eos/lhcb/wg/PID/PIDCalib_2015_electrons/pidcalib_BJpsiEE_MD.root'],
-  'MagDown_2016': ['/eos/lhcb/wg/PID/PIDCalib_2016_electrons/pidcalib_BJpsiEE_MD_TAGCUT.root'],
-  'MagDown_2017': ['/eos/lhcb/wg/PID/PIDCalib_2017_electrons/Turbo2017_B2KJpsiEE_MagDown.root'],
-  'MagDown_2018': ['/eos/lhcb/wg/PID/PIDCalib_2018_electrons/Turbo2018_B2KJpsiEE_MagDown.root'],
-  'MagUp_2015'  : ['/eos/lhcb/wg/PID/PIDCalib_2015_electrons/pidcalib_BJpsiEE_MU.root'],
-  'MagUp_2016'  : ['/eos/lhcb/wg/PID/PIDCalib_2016_electrons/pidcalib_BJpsiEE_MU_TAGCUT.root'], 
-  'MagUp_2017'  : ['/eos/lhcb/wg/PID/PIDCalib_2017_electrons/Turbo2017_B2KJpsiEE_MagUp.root'], 
-  'MagUp_2018'  : ['/eos/lhcb/wg/PID/PIDCalib_2018_electrons/Turbo2018_B2KJpsiEE_MagUp.root'], 
+  'MagDown_2015': ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2015_electrons/pidcalib_BJpsiEE_MD.root'],
+  'MagDown_2016': ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2016_electrons/pidcalib_BJpsiEE_MD_TAGCUT.root'],
+  'MagDown_2017': ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2017_electrons/Turbo2017_B2KJpsiEE_MagDown.root'],
+  'MagDown_2018': ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2018_electrons/Turbo2018_B2KJpsiEE_MagDown.root'],
+  'MagUp_2015'  : ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2015_electrons/pidcalib_BJpsiEE_MU.root'],
+  'MagUp_2016'  : ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2016_electrons/pidcalib_BJpsiEE_MU_TAGCUT.root'], 
+  'MagUp_2017'  : ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2017_electrons/Turbo2017_B2KJpsiEE_MagUp.root'], 
+  'MagUp_2018'  : ['root://eoslhcb.cern.ch//eos/lhcb/wg/PID/PIDCalib_2018_electrons/Turbo2018_B2KJpsiEE_MagUp.root'], 
 }
 
 converted_run1_pidcalib_datasets_dstar_k = {
@@ -64,3 +65,20 @@ converted_run1_pidcalib_datasets_dstar_pi = {
   "MagDown_2011" : [f"{run1_dir}/MagDown/DSt_Pi_MagDown_Strip21r1_{i}.root" for i in range(35) ], 
   "MagDown_2012" : [f"{run1_dir}/MagDown/DSt_Pi_MagDown_Strip21_{i}.root" for i in range(71) ], 
 }
+
+converted_run1_pidcalib_datasets_lam0_p = {
+  "MagUp_2011"   : [f"{run1_dir}/MagUp/Lam0_P_MagUp_Strip21r1_{i}.root" for i in range(30) ], 
+  "MagUp_2012"   : [f"{run1_dir}/MagUp/Lam0_P_MagUp_Strip21_{i}.root" for i in range(119) ], 
+  "MagDown_2011" : [f"{run1_dir}/MagDown/Lam0_P_MagDown_Strip21r1_{i}.root" for i in range(41) ], 
+  "MagDown_2012" : [f"{run1_dir}/MagDown/Lam0_P_MagDown_Strip21_{i}.root" for i in range(121) ], 
+}
+
+photon_run1_datasets_kstargamma = {
+  "2011-2012" : [f"{photon_dir}/sWtd_run0_mode0.root"], 
+}
+
+photon_run2_datasets_kstargamma = {
+  "2016" : [f"{photon_dir}/sWtd_run1_mode0.root"], 
+  "2017" : [f"{photon_dir}/sWtd_run2_mode0.root"], 
+  "2018" : [f"{photon_dir}/sWtd_run3_mode0.root"], 
+}
