@@ -10,7 +10,7 @@
 ###############################################################################
 
 variable = {
-    "branch" : "probe_PIDp", 
+    "expression" : "probe_PIDp", 
     "data_range" : (-150., 150.), 
     "transform_forward" : "x", 
     "transform_backward" : "x", 
