@@ -10,7 +10,8 @@
 ###############################################################################
 
 variable = {
-    "branch" : "probe_MC15TuneV1_ProbNNpi", 
+    "expression" : "probe_MC15TuneV1_ProbNNpi", 
+    "mc_expression" : "{prefix}_MC15TuneV1_ProbNNpi" ,
     "data_range" : (0., 1.), 
     "transform_forward" : "1.-(1.-x**0.25)**0.5", 
     "transform_backward" : "(1.-(1.-x)**2)**4", 
